#include "property.h"
#include <stdio.h>
#include <string.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include "aswm/mapper/mapper.h"

void OnPropertyNotify(Aswm* aswm, XPropertyEvent* e) {
	printf("Property change for window %lu:\n", e->window);
	char* property_name = XGetAtomName(e->display, e->atom);
	printf("\t%s\n", property_name);
	if(strcmp(property_name, "_NET_WM_NAME") == 0) {
		printf("\t-> Unhandled\n");
		XTextProperty wm_name_property;
		char** wm_name;
		int wm_name_count;
		XGetTextProperty(e->display, e->window, &wm_name_property, e->atom);
		Xutf8TextPropertyToTextList(e->display, &wm_name_property, &wm_name, &wm_name_count);
		for(auto i = 0; i < wm_name_count; i++)
			printf("\t_NET_WM_NAME: %s\n", wm_name[i]);
		XFreeStringList(wm_name);
	} else if(strcmp(property_name, "WM_NAME") == 0) {
		printf("\t-> Unhandled\n");
		XTextProperty text_property;
		if(XGetWMName(e->display, e->window, &text_property)) {
			printf("\tWM_NAME: %s\n", text_property.value);
		}
	} else if(strcmp(property_name, "WM_NORMAL_HINTS") == 0) {
		XSizeHints* size = XAllocSizeHints();
			long supplied;
			if(XGetWMNormalHints(e->display, e->window, size, &supplied)) {
				printf("\tWMNormalHints:\n");
				printf("\t\tMin w:%i\n", size->min_width);
				printf("\t\tMin h:%i\n", size->min_height);
				printf("\t\tMax w:%i\n", size->max_width);
				printf("\t\tMax h:%i\n", size->max_height);
				printf("\t\tBase w:%i\n", size->base_width);
				printf("\t\tBase h:%i\n", size->base_height);
				aswm_handle_normal_hints(
						aswm->display, aswm->current_desktop.stack_root,
						e->window, size, supplied);
			}
			XFree(size);
	} else if(strcmp(property_name, "WM_HINTS") == 0) {
		printf("\t-> Unhandled\n");
		XWMHints* hints = XGetWMHints(aswm->display, e->window);
		printf("\t\tflags Input: %li\n", hints->flags & InputHint);
		printf("\t\tflags Input: %li\n", hints->flags & StateHint);
		printf("\t\tflags IconPixmap: %li\n", hints->flags & IconPixmapHint);
		printf("\t\tflags WindowHint: %li\n", hints->flags & IconWindowHint);
		printf("\t\tflags IconPosition: %li\n", hints->flags & IconPositionHint);
		printf("\t\tflags IconMask: %li\n", hints->flags & IconMaskHint);
		printf("\t\tflags WindowGroup: %li\n", hints->flags & WindowGroupHint);
		printf("\t\tflags XUrgency: %li\n", hints->flags & XUrgencyHint);
		printf("\t\tflags AllHints: %li\n", hints->flags & AllHints);
		printf("\t\tinitial_state: %i\n", hints->initial_state);
		XFree(hints);
	} else if(strcmp(property_name, "WM_PROTOCOLS") == 0) {
		printf("\t-> Unhandled\n");
		Atom* protocols;
		int count;
		XGetWMProtocols(aswm->display, e->window, &protocols, &count);
		if(count > 0)
			printf("\t\t");
		for(auto i = 0; i < count; i++) {
			char* name = XGetAtomName(aswm->display, protocols[i]);
			printf("%s ", name);
			XFree(name);
		}
		if(count > 0)
			printf("\n");
		XFree(protocols);

	} else {
		printf("\t-> Unhandled\n");
	}
	XFree(property_name);
}
